/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 reception library
* Version		: v1.2
* Last updated	: 21.04.07
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/

#include "lib_dmx_in.h"

// ********************* local definitions *********************

enum {IDLE, BREAK, STARTB, STARTADR};			//DMX states

		 uint8_t 	 gDmxState;
		 uint8_t 	*gDmxPnt;
		 uint16_t	 DmxCount;


// *************** DMX Reception Initialisation ****************
void init_DMX(void)
{
#ifdef USE_DIP
DDRB &= ~((1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6));				//set up DIPs
PORTB |= (1<<2)|(1<<3)|(1<<4)|(1<<5)|(1<<6);
DDRD &= ~((1<<5)|(1<<6)|(1<<7));
PORTD |= (1<<5)|(1<<6)|(1<<7);
PORTD |= (1<<4); 							//Internen Pull-Up fr RX Optokoppler an
#endif


UBRRH  = 0;
UBRRL  = ((F_OSC/4000)-1);						//250kbaud, 8N2
UCSRC |= (3<<UCSZ0)|(1<<USBS);
UCSRB |= (1<<RXEN)|(1<<RXCIE);

gDmxState= IDLE;

uint8_t i;
for (i=0; i<DMX_CHANNELS; i++)
	{
	DmxField[i]= 0;
	}

}


// ************* get DMX start address **************
void get_dips(void)
{
#ifdef USE_DIP
uint16_t Temp=0;							//Variable fr Startadresse

if (!(PINB &(1<<2)))
	{
	Temp= Temp +1;							//1st bit
	}
if (!(PINB &(1<<3)))
	{
	Temp= Temp +2;							//2nd bit
	}
if (!(PINB &(1<<4)))
	{
	Temp= Temp +4;							//3rd bit
	}
if (!(PINB &(1<<5)))
	{
	Temp= Temp +8;							//4th bit
	}
if (!(PINB &(1<<6)))
	{
	Temp= Temp +16;							//5th bit
	}
if (!(PIND &(1<<5)))
	{
	Temp= Temp +32;							//6th bit
	}
if (!(PIND &(1<<6)))
	{
	Temp= Temp +64;							//7th bit
	}
if (!(PIND &(1<<7)))
	{
	Temp= Temp +128;						//8th bit
	}


if (Temp != 0)
	{
	DmxAddress= Temp;
	if (!(UCSRB &(1<<RXCIE)))					//if receiver was disabled -> enable and wait for break
		{
		gDmxState= IDLE;
		UCSRB |= (1<<RXCIE);
		}
	}
else
	{
	UCSRB &= ~(1<<RXCIE);						//disable Receiver if start address = 0
	for (Temp=0; Temp<DMX_CHANNELS; Temp++)
		{
		DmxField[Temp]= 0;
		}
	}
#endif 
}


// *************** DMX Reception ISR ****************
ISR (USART_RX_vect)
{
uint8_t USARTstate= UCSRA;						//get state
uint8_t DmxByte= UDR;							//get data
uint8_t DmxState= gDmxState;						//just get once from SRAM!!!

 
if (USARTstate &(1<<FE))						//check for break
	{
	UCSRA &= ~(1<<FE);							//reset flag
	DmxCount= DmxAddress;						//reset frame counter
	gDmxState= BREAK;
	}

else if (DmxState == BREAK)
	{
	if (DmxByte == 0) 
		{
		gDmxState= STARTB;					//normal start code detected
		gDmxPnt= ((uint8_t*)DmxField +1);
		}
	else gDmxState= IDLE;
	}
	
else if (DmxState == STARTB)
	{
	if (--DmxCount == 0)						//start address reached?
		{
		gDmxState= STARTADR;
		DmxField[0]= DmxByte;
		}
	}

else if (DmxState == STARTADR)
	{
	uint8_t *DmxPnt;
	DmxPnt= gDmxPnt;
	*DmxPnt= DmxByte;
	if (++DmxPnt >= (DmxField +DMX_CHANNELS)) 	//all ch received?
		{
		gDmxState= IDLE;
		}
	else gDmxPnt= DmxPnt;
	}							
}


